@@ -13,7 +13,7 @@ from . import __version__
 from .projects import ChildProject
 from .converters import *
 from .tables import IndexTable, IndexColumn, assert_dataframe, assert_columns_presence
-from .utils import Segment, intersect_ranges, path_is_parent
+from .utils import Segment, intersect_ranges, path_is_parent, TimeInterval, series_to_datetime
 
 
 class AnnotationManager:
@@ -60,7 +60,7 @@ class AnnotationManager:
         ),
         IndexColumn(
             name="filter",
-            description="source file to filter in (for rttm and alice only)",
+            description="source file to target. this field is dedicated to rttm and ALICE annotations that may combine annotations from several recordings into one same text file.",
             required=False,
         ),
         IndexColumn(
@@ -73,7 +73,7 @@ class AnnotationManager:
         IndexColumn(
             name="imported_at",
             description="importation date (automatic column, don't specify)",
-            datetime="%Y-%m-%d %H:%M:%S",
+            datetime={"%Y-%m-%d %H:%M:%S"},
             required=False,
             generated=True,
         ),
@@ -90,6 +90,12 @@ class AnnotationManager:
             required=False,
             generated=True,
         ),
+        IndexColumn(
+            name="merged_from",
+            description="sets used to generate this annotation by merging (comma separated)",
+            required=False,
+            generated=True,
+        ),
     ]
 
     SEGMENTS_COLUMNS = [
@@ -125,8 +131,8 @@ class AnnotationManager:
         ),
         IndexColumn(
             name="vcm_type",
-            description="vocal maturity defined as: C (canonical), N (non-canonical), Y (crying) L (laughing), J (junk)",
-            choices=["C", "N", "Y", "L", "J", "NA"],
+            description="vocal maturity defined as: C (canonical), N (non-canonical), Y (crying) L (laughing), J (junk), U (uncertain)",
+            choices=["C", "N", "Y", "L", "J", "U","NA"],
         ),
         IndexColumn(
             name="lex_type",
@@ -140,13 +146,18 @@ class AnnotationManager:
         ),
         IndexColumn(
             name="msc_type",
-            description="morphosyntactical complexity of the utterances defined as: 0 (0 meaningful word), 1 (1 meaningful word), S (simple utterance), C (complex utterance)",
-            choices=["0", "1", "S", "C"],
+            description="morphosyntactical complexity of the utterances defined as: 0 (0 meaningful word), 1 (1 meaningful word), 2 (2 meaningful words), S (simple utterance), C (complex utterance), U (uncertain)",
+            choices=["0", "1", "2", "S", "C", "U"],
+        ),
+        IndexColumn(
+            name="gra_type",
+            description="grammaticality of the utterances defined as: G (grammatical), J (ungrammatical), U (uncertain)",
+            choices=["G", "J", "U"],
         ),
         IndexColumn(
             name="addressee",
-            description="T if target-child-directed, C if other-child-directed, A if adult-directed, U if uncertain or other. Multiple values should be sorted and separated by commas",
-            choices=["T", "C", "A", "U", "NA"],
+            description="T if target-child-directed, C if other-child-directed, A if adult-directed, O if addressed to other, P if addressed to a pet, U if uncertain or other. Multiple values should be sorted and separated by commas",
+            choices=["T", "C", "A", "O", "P", "U", "NA"],
         ),
         IndexColumn(
             name="transcription", description="orthographic transcription of the speach"
@@ -308,6 +319,8 @@ class AnnotationManager:
                     for dup in duplicates.to_dict(orient="records")
                 ]
             )
+        
+        warnings += self._check_for_outdated_merged_sets()
 
         return errors, warnings
 
@@ -375,7 +388,7 @@ class AnnotationManager:
         """Update the annotations index,
         while enforcing its good shape.
         """
-        self.annotations[["time_seek", "range_onset", "range_offset"]].fillna(
+        self.annotations.loc[:,["time_seek", "range_onset", "range_offset"]].fillna(
             0, inplace=True
         )
         self.annotations[
@@ -384,14 +397,46 @@ class AnnotationManager:
         self.annotations.to_csv(
             os.path.join(self.project.path, "metadata/annotations.csv"), index=False
         )
+        
+    def _check_for_outdated_merged_sets(self, sets: set = None):
+        """Checks the annotations dataframe for sets that were used in merged sets and modified afterwards.
+        This method produces warnings and suggestions to update the considered merged sets.
+        
+        :param sets: names of the original sets (sets used to merge) to consider.
+        :type sets: set
+        :return: List of warnings to give regarding the presence of outdated merged sets
+        :rtype: List[str]
+        """
+        warnings = []
+        
+        #make a copy of annotation index, keep only the last modification date for each set (will not detect specific cases like partial merge)
+        df = self.annotations.copy().sort_values(['set','imported_at']).groupby(['set']).last()
+        
+        #build a dictionary capturing the last modification date for each set.
+        last_modif = {} 
+        for i, row in df.iterrows():
+            last_modif[i] = row['imported_at']
+        
+        #iterate through sets that were built from a merge and compare their last modification date to the one of their original set.
+        merged_sets = df.dropna(subset=['merged_from'])[['merged_from', 'imported_at']]      
+        for i, row in merged_sets.iterrows():
+            for j in row['merged_from'].split(','):
+                #if a list of sets was given and the set is not in that list, skip it
+                if (sets is not None and j in sets) or sets is None:
+                    if row['imported_at'] < last_modif[j]:
+                        warnings.append("set {} is outdated because the {} set it is merged from was modified. Consider updating or rerunning the creation of the {} set.".format(i,j,i))
+                        
+        return warnings
 
     def _import_annotation(
-        self, import_function: Callable[[str], pd.DataFrame], annotation: dict
+        self, import_function: Callable[[str], pd.DataFrame], params: dict, annotation: dict
     ):
         """import and convert ``annotation``. This function should not be called outside of this class.
 
         :param import_function: If callable, ``import_function`` will be called to convert the input annotation into a dataframe. Otherwise, the conversion will be performed by a built-in function.
         :type import_function: Callable[[str], pd.DataFrame]
+        :param params: Optional parameters. With ```new_tiers```, the corresponding EAF tiers will be imported
+        :type params: dict
         :param annotation: input annotation dictionary (attributes defined according to :ref:`ChildProject.annotations.AnnotationManager.SEGMENTS_COLUMNS`)
         :type annotation: dict
         :return: output annotation dictionary (attributes defined according to :ref:`ChildProject.annotations.AnnotationManager.SEGMENTS_COLUMNS`)
@@ -427,7 +472,7 @@ class AnnotationManager:
                 df = import_function(path)
             elif annotation_format in converters:
                 converter = converters[annotation_format]
-                df = converter.convert(path, filter)
+                df = converter.convert(path, filter, **params)
             else:
                 raise ValueError(
                     "file format '{}' unknown for '{}'".format(annotation_format, path)
@@ -488,6 +533,7 @@ class AnnotationManager:
         input: pd.DataFrame,
         threads: int = -1,
         import_function: Callable[[str], pd.DataFrame] = None,
+        new_tiers: list = None,
     ) -> pd.DataFrame:
         """Import and convert annotations.
 
@@ -497,6 +543,8 @@ class AnnotationManager:
         :type threads: int, optional
         :param import_function: If specified, the custom ``import_function`` function will be used to convert all ``input`` annotations, defaults to None
         :type import_function: Callable[[str], pd.DataFrame], optional
+        :param new_tiers: List of EAF tiers names. If specified, the corresponding EAF tiers will be imported.
+        :type new_tiers: list[str], optional
         :return: dataframe of imported annotations, as in :ref:`format-annotations`.
         :rtype: pd.DataFrame
         """
@@ -536,12 +584,12 @@ class AnnotationManager:
 
         if threads == 1:
             imported = input.apply(
-                partial(self._import_annotation, import_function), axis=1
+                partial(self._import_annotation, import_function, {"new_tiers": new_tiers}), axis=1
             ).to_dict(orient="records")
         else:
             with mp.Pool(processes=threads if threads > 0 else mp.cpu_count()) as pool:
                 imported = pool.map(
-                    partial(self._import_annotation, import_function),
+                    partial(self._import_annotation, import_function, {"new_tiers": new_tiers}),
                     input.to_dict(orient="records"),
                 )
 
@@ -555,6 +603,11 @@ class AnnotationManager:
         self.read()
         self.annotations = pd.concat([self.annotations, imported], sort=False)
         self.write()
+        
+        sets = set(input['set'].unique())
+        outdated_sets = self._check_for_outdated_merged_sets(sets= sets)
+        for warning in outdated_sets:
+            print("warning: {}".format(warning))
 
         return imported
 
@@ -617,6 +670,10 @@ class AnnotationManager:
 
         self.annotations = self.annotations[self.annotations["set"] != annotation_set]
         self.write()
+        
+        outdated_sets = self._check_for_outdated_merged_sets(sets= {annotation_set})
+        for warning in outdated_sets:
+            print("warning: {}".format(warning))
 
     def rename_set(
         self,
@@ -693,13 +750,45 @@ class AnnotationManager:
         self.write()
 
     def merge_annotations(
-        self, left_columns, right_columns, columns, output_set, input
+        self, left_columns, right_columns, columns, output_set, input, skip_existing: bool = False
     ):
+        """From 2 DataFrames listing the annotation indexes to merge together (those indexes should come from
+        the intersection of the left_set and right_set indexes), the listing of the columns
+        to merge and name of the output_set, creates the resulting csv files containing the converted merged
+        segments and returns the new indexes to add to annotations.csv.
+
+        :param left_columns: list of the columns to include from the left set
+        :type left_columns: list[str]
+        :param right_columns: list of the columns to include from the right set
+        :type right_columns: list[str]
+        :param columns: additional columns to add to the segments, key is the column name
+        :type columns: dict
+        :param output_set: name of the set to save the new merged files into
+        :type output_set: str
+        :param input: annotation indexes to use for the merge, contains keys 'left_annotations' and 'right_annotations' to separate indexes from left and right set
+        :type input: dict
+        :param input:
+        :type input: bool
+        :return: annotation indexes created by the merge, should be added to annotations.csv
+        :rtype: pandas.DataFrame
+        """
+        #get the left and right annotation dataframes
         left_annotations = input["left_annotations"]
         right_annotations = input["right_annotations"]
-
+        
+        #start the new annotations from a copy of the left set
         annotations = left_annotations.copy()
+        #store the annotation filenames used to keep a reference to those existing files
+        annotations['left_annotation_filename'] = annotations["annotation_filename"]
+        annotations['right_annotation_filename'] = right_annotations['annotation_filename']
+        #populate the raw_filename column with the raw filenames of the sets used to merge, separated by a comma
+        annotations['raw_filename'] = left_annotations['raw_filename'] + ',' + right_annotations['raw_filename']
+        #package version is the version used by the merge, not the one used in the merged sets
+        annotations["package_version"] = __version__
+        
+        #format of a merged set is undefined
         annotations["format"] = ""
+        #compute the names of the new annotation filenames that will be created
         annotations["annotation_filename"] = annotations.apply(
             lambda annotation: "{}_{}_{}.csv".format(
                 os.path.splitext(annotation["recording_filename"])[0],
@@ -708,12 +797,23 @@ class AnnotationManager:
             ),
             axis=1,
         )
-
+        #store in 'merged_from' the names of the sets it was merged from
+        annotations['merged_from'] = ','.join(np.concatenate([left_annotations['set'].unique() , right_annotations['set'].unique()]))
+        #the timestamps will be recomputed from the start of the file, so time_seek is always 0 on a merged set
+        annotations['time_seek'] = 0
+        
+        #if skip existing, only keep the line where the resulting converted file does not already exist (even as a broken symlink)
+        if skip_existing:
+            annotations = annotations[~annotations['annotation_filename'].map(lambda x : os.path.lexists(os.path.join(self.project.path,"annotations",output_set, "converted", x)))]
+            left_annotations = left_annotations[left_annotations['annotation_filename'].isin(annotations['left_annotation_filename'].to_list())]
+            right_annotations = right_annotations[right_annotations['annotation_filename'].isin(annotations['right_annotation_filename'].to_list())]
+        
         for key in columns:
             annotations[key] = columns[key]
 
         annotations["set"] = output_set
 
+        #check the presence of the converted files in the left_set
         left_annotation_files = [
             os.path.join(
                 self.project.path,
@@ -728,6 +828,7 @@ class AnnotationManager:
             f for f in left_annotation_files if not os.path.exists(f)
         ]
 
+        #check the presence of the converted files in the right_set
         right_annotation_files = [
             os.path.join(
                 self.project.path,
@@ -756,6 +857,7 @@ class AnnotationManager:
                 )
             )
 
+        #get the actual annotation segments
         left_segments = self.get_segments(left_annotations)
         right_segments = self.get_segments(right_annotations)
 
@@ -771,6 +873,7 @@ class AnnotationManager:
             right_segments.columns.union(rc, sort=False), axis=1, fill_value="NA"
         )
 
+        #merge left and right annotations segments
         output_segments = left_segments[list(lc)].merge(
             right_segments[list(rc)],
             how="outer",
@@ -784,34 +887,28 @@ class AnnotationManager:
         output_segments["segment_offset"] = (
             output_segments["segment_offset"].fillna(0).astype(int)
         )
-
-        output_segments["raw_filename"] = (
-            output_segments["raw_filename_x"] + "," + output_segments["raw_filename_y"]
-        )
-
-        annotations.drop(columns="raw_filename", inplace=True)
-        annotations = annotations.merge(
-            output_segments[["interval", "raw_filename"]].dropna().drop_duplicates(),
-            how="left",
-            left_on="interval",
-            right_on="interval",
-        )
-        annotations.rename(columns={"raw_filename": "raw_filename"}, inplace=True)
-        annotations["generated_at"] = datetime.datetime.now().strftime(
-            "%Y-%m-%d %H:%M:%S"
-        )
-
+        
         output_segments["raw_filename"] = (
             output_segments["raw_filename_x"].fillna("")
-            + ","
+            + "," 
             + output_segments["raw_filename_y"].fillna("")
         )
+        output_segments["raw_filename"] = output_segments["raw_filename"].str.strip(',')
         output_segments.drop(
             columns=["raw_filename_x", "raw_filename_y", "time_seek"], inplace=True
         )
 
+        #drop unused columns, get the currect datetime and store it in imported_at
+        annotations.drop(columns=['right_annotation_filename', 'left_annotation_filename'], inplace=True)
+        annotations["imported_at"] = datetime.datetime.now().strftime(
+            "%Y-%m-%d %H:%M:%S"
+        )
+
+        
+
         output_segments.fillna("NA", inplace=True)
 
+        #create the new converted files from the merged annotation segments
         for annotation in annotations.to_dict(orient="records"):
             interval = annotation["interval"]
             annotation_filename = annotation["annotation_filename"]
@@ -857,12 +954,16 @@ class AnnotationManager:
         left_columns: List[str],
         right_columns: List[str],
         output_set: str,
+        full_set_merge: bool = True,
+        skip_existing: bool = False,
         columns: dict = {},
+        recording_filter: str = None,
         threads=-1,
     ):
         """Merge columns from ``left_set`` and ``right_set`` annotations, 
         for all matching segments, into a new set of annotations named
-        ``output_set``.
+        ``output_set`` that will be saved in the dataset. ``output_set``
+        must not already exist if full_set_merge is True. 
 
         :param left_set: Left set of annotations.
         :type left_set: str
@@ -874,9 +975,23 @@ class AnnotationManager:
         :type right_columns: List
         :param output_set: Name of the output annotations set.
         :type output_set: str
+        :param full_set_merge: The merge is meant to create the entired merged set. Therefore, the set should not already exist. defaults to True
+        :type full_set_merge: bool
+        :param skip_existing: The merge will skip already existing lines in the merged set. So both the annotation index and resulting converted csv will not change for those lines
+        :type skip_existing: bool
+        :param columns: Additional columns to add to the resulting converted annotations.
+        :type columns: dict
+        :param recording_filter: set of recording_filenames to merge.
+        :type recording_filter: set[str]
+        :param threads: number of threads
+        :type threads: int
         :return: [description]
         :rtype: [type]
         """
+        existing_sets = self.annotations['set'].unique()
+        if full_set_merge: assert output_set not in existing_sets, "output_set <{}> already exists, remove the existing set or another name.".format(output_set)
+        assert left_set in existing_sets, "left_set <{}> was not found, check the spelling.".format(left_set)
+        assert right_set in existing_sets, "right_set <{}> was not found, check the spelling.".format(right_set)
         assert left_set != right_set, "sets must differ"
         assert not (
             set(left_columns) & set(right_columns)
@@ -904,6 +1019,9 @@ class AnnotationManager:
             self.annotations["set"].isin([left_set, right_set])
         ]
         annotations = annotations[annotations["error"].isnull()]
+        
+        if recording_filter:
+            annotations = annotations[annotations['recording_filename'].isin(recording_filter)]
 
         intersection = AnnotationManager.intersection(
             annotations, sets=[left_set, right_set]
@@ -937,7 +1055,7 @@ class AnnotationManager:
         pool = mp.Pool(processes=threads if threads > 0 else mp.cpu_count())
         annotations = pool.map(
             partial(
-                self.merge_annotations, left_columns, right_columns, columns, output_set
+                self.merge_annotations, left_columns, right_columns, columns, output_set, skip_existing=skip_existing
             ),
             input_annotations,
         )
@@ -951,7 +1069,8 @@ class AnnotationManager:
         annotations.fillna({"raw_filename": "NA"}, inplace=True)
 
         self.read()
-        self.annotations = pd.concat([self.annotations, annotations], sort=False)
+        # if annotations.csv can have duplicate entries with same converted filename and is normal, check this https://stackoverflow.com/a/45927402 and change the code
+        self.annotations = pd.concat([self.annotations, annotations], sort=False).drop_duplicates(subset=['set','recording_filename','annotation_filename'], keep='last')
         self.write()
 
     def get_segments(self, annotations: pd.DataFrame) -> pd.DataFrame:
@@ -1186,24 +1305,22 @@ class AnnotationManager:
         return pd.concat(stack) if len(stack) else pd.DataFrame()
 
     def get_within_time_range(
-        self, annotations: pd.DataFrame, start_time: str, end_time: str, errors="raise"
+        self, annotations: pd.DataFrame, interval : TimeInterval, errors="raise"
     ):
-        """Clip all input annotations within a given HH:MM clock-time range.
+        """Clip all input annotations within a given HH:MM:SS clock-time range.
         Those that do not intersect the input time range at all are filtered out.
 
         :param annotations: DataFrame of input annotations to filter. The only columns that are required are: ``recording_filename``, ``range_onset``, and ``range_offset``.
         :type annotations: pd.DataFrame
-        :param start: onset HH:MM clocktime
-        :type start: str
-        :param end: offset HH:MM clocktime
-        :type end: str
+        :param interval: Interval of hours to consider, contains the start hour and end hour
+        :type interval: TimeInterval
         :param errors: how to deal with invalid start_time values for the recordings. Takes the same values as ``pandas.to_datetime``.
         :type errors: str
-        :return: a DataFrame of annotations;
-        For each row, ``range_onset`` and ``range_offset`` are clipped within the desired clock-time range.
-        The clock-time corresponding to the onset and offset of each annotation
-        is stored in two newly created columns named ``range_onset_time`` and ``range_offset_time``.
-        If the input annotation exceeds 24 hours, one row per matching interval is returned.
+        :return: a DataFrame of annotations; \
+        For each row, ``range_onset`` and ``range_offset`` are clipped within the desired clock-time range. \
+        The clock-time corresponding to the onset and offset of each annotation \
+        is stored in two newly created columns named ``range_onset_time`` and ``range_offset_time``. \
+        If the input annotation exceeds 24 hours, one row per matching interval is returned. \
         :rtype: pd.DataFrame
         """
 
@@ -1217,30 +1334,19 @@ class AnnotationManager:
         def get_ms_since_midight(dt):
             return (dt - dt.replace(hour=0, minute=0, second=0)).total_seconds() * 1000
 
-        try:
-            start_dt = datetime.datetime.strptime(start_time, "%H:%M")
-        except:
-            raise ValueError(
-                f"invalid value for start_time ('{start_time}'); should have HH:MM format instead"
-            )
+        #assert end_dt > start_dt, "end_time must follow start_time"
+        # no reason to keep this condition, 23:00 to 03:00 is completely acceptable
 
-        try:
-            end_dt = datetime.datetime.strptime(end_time, "%H:%M")
-        except:
-            raise ValueError(
-                f"invalid value for end_time ('{end_time}'); should have HH:MM format instead"
-            )
-
-        assert end_dt > start_dt, "end_time must follow start_time"
-
-        start_ts = get_ms_since_midight(start_dt)
-        end_ts = get_ms_since_midight(end_dt)
+        if not isinstance(interval, TimeInterval): raise ValueError("interval must be a TimeInterval object")
+        start_ts = get_ms_since_midight(interval.start)
+        end_ts = get_ms_since_midight(interval.stop)
 
         annotations = annotations.merge(
             self.project.recordings[["recording_filename", "start_time"]], how="left"
         )
-        annotations["start_time"] = pd.to_datetime(
-            annotations["start_time"], format="%H:%M", errors=errors
+        
+        annotations['start_time'] = series_to_datetime(
+                annotations['start_time'], ChildProject.RECORDINGS_COLUMNS, 'start_time'
         )
 
         # remove values with NaT start_time
@@ -1262,8 +1368,18 @@ class AnnotationManager:
 
         matches = []
         for annotation in annotations.to_dict(orient="records"):
+            #onsets = np.arange(start_ts, annotation["range_offset_ts"], 86400 * 1000)
+            #offsets = onsets + (end_ts - start_ts)
+            
             onsets = np.arange(start_ts, annotation["range_offset_ts"], 86400 * 1000)
-            offsets = onsets + (end_ts - start_ts)
+            offsets = np.arange(end_ts, annotation["range_offset_ts"], 86400 * 1000)
+            #treat edge cases when the offset is after the end of annotation, onset before start etc
+            if len(onsets) > 0 and onsets[0] < annotation["range_onset_ts"] :
+                if len(offsets) > 0 and offsets[0] < annotation["range_onset_ts"]: onsets = onsets[1:]
+                else : onsets[0] = annotation["range_onset_ts"]
+            if len(offsets) > 0 and offsets[0] < annotation["range_onset_ts"] : offsets = offsets[1:]
+            if len(onsets) > 0 and len(offsets) > 0 and onsets[0] > offsets[0] : onsets = np.append(annotation["range_onset_ts"], onsets)
+            if (len(onsets) > 0 and len(offsets) > 0 and onsets[-1] > offsets[-1]) or len(onsets) > len(offsets) : offsets = np.append(offsets,annotation["range_offset_ts"])
 
             xs = (Segment(onset, offset) for onset, offset in zip(onsets, offsets))
             ys = iter(
@@ -1313,10 +1429,10 @@ class AnnotationManager:
         :type onset: str, optional
         :param offset: column storing the offset timestamp in milliseconds, defaults to "segment_offset"
         :type offset: str, optional
-        :return: Returns the input dataframe with two new columns ``onset_time`` and ``offset_time``.
-        ``onset_time`` is a datetime object corresponding to the onset of the segment.
-        ``offset_time`` is a datetime object corresponding to the offset of the segment.
-        In case either ``start_time`` or ``date_iso`` is not specified for the corresponding recording,
+        :return: Returns the input dataframe with two new columns ``onset_time`` and ``offset_time``. \
+        ``onset_time`` is a datetime object corresponding to the onset of the segment. \
+        ``offset_time`` is a datetime object corresponding to the offset of the segment. \
+        In case either ``start_time`` or ``date_iso`` is not specified for the corresponding recording, \
         both values will be set to NaT.
         :rtype: pd.DataFrame
         """
@@ -1345,19 +1461,12 @@ class AnnotationManager:
         )
 
         if ignore_date:
-            segments["start_time"] = pd.to_datetime(
-                segments["start_time"], format="%H:%M", errors="coerce",
+            segments['start_time'] = series_to_datetime(
+                segments['start_time'], ChildProject.RECORDINGS_COLUMNS, 'start_time'
             )
         else:
-            segments["start_time"] = pd.to_datetime(
-                segments[["date_iso", "start_time"]].apply(
-                    lambda row: "{} {}".format(
-                        str(row["date_iso"]), str(row["start_time"])
-                    ),
-                    axis=1,
-                ),
-                format="%Y-%m-%d %H:%M",
-                errors="coerce",
+            segments['start_time'] = series_to_datetime(
+                segments['start_time'], ChildProject.RECORDINGS_COLUMNS, 'start_time', date_series = segments['date_iso'], date_index_list = ChildProject.RECORDINGS_COLUMNS, date_column_name = 'date_iso'
             )
 
         segments["onset_time"] = segments["start_time"] + pd.to_timedelta(
