@@ -6,15 +6,23 @@ import math
 import multiprocessing as mp
 import os
 import pandas as pd
-from panoptes_client import Panoptes, Project, Subject, SubjectSet, Classification
 import shutil
 import subprocess
 import sys
+import array
 import traceback
 from typing import List
 from yaml import dump
 
+import matplotlib.pyplot as plt
+from matplotlib.ticker import ScalarFormatter
+from numpy import log10
+
 from pydub import AudioSegment
+from pydub.utils import get_array_type
+
+from parselmouth import Sound
+from parselmouth import SpectralAnalysisWindowShape
 
 import ChildProject
 from ChildProject.pipelines.pipeline import Pipeline
@@ -22,6 +30,8 @@ from ChildProject.tables import assert_dataframe, assert_columns_presence
 
 from typing import Tuple
 
+import time
+
 
 def pad_interval(
     onset: int, offset: int, chunks_length: int, chunks_min_amount: int = 1
@@ -94,6 +104,42 @@ class ZooniversePipeline(Pipeline):
         return (self.zooniverse_login, self.zooniverse_pwd)
 
     def _split_recording(self, segments: pd.DataFrame) -> list:
+        
+        #from raw sound data and sampling rate, build the spectrogram as a matplotlib figure and return it
+        def _create_spectrogram(data,sr):
+            snd = Sound(data,sampling_frequency=sr)
+            # this parameters were chosen to output a spectrogram useful for zooniverse applications (short sounds from babies) we did not feel the need to have flexibility on them
+            spectrogram = snd.to_spectrogram(window_length=0.0075,maximum_frequency=8000, time_step= 0.0001 ,frequency_step = 0.1,window_shape= SpectralAnalysisWindowShape.GAUSSIAN)
+            
+            fig = plt.figure(figsize=(12, 6.75)) #size of the image, we chose 1200x675 pixels for a better display on zooniverse
+            gs = fig.add_gridspec(2, hspace=0, height_ratios=[1, 3]) #2 plots (spectrogram 3x bigger than oscillogram)
+            axs = gs.subplots(sharex=True)
+            
+            #scpectrogram plot
+            dynamic_range=65
+            X, Y = spectrogram.x_grid(), spectrogram.y_grid()
+            sg_db = 10 * log10(spectrogram.values)
+            axs[1].pcolormesh(X, Y, sg_db, vmin=sg_db.max() - dynamic_range, cmap='Greys')
+            axs[1].set_ylim([spectrogram.ymin, spectrogram.ymax])
+            axs[1].set_xlabel("time [s]")
+            axs[1].set_ylabel("frequency [Hz]")
+            axs[1].tick_params( labelright=True)
+            axs[1].set_xlim([snd.xmin, snd.xmax])
+            
+            #oscillogram plot
+            axs[0].plot(snd.xs(), snd.values.T, linewidth=0.5)
+            axs[0].set_xlim([snd.xmin, snd.xmax])
+            axs[0].set_ylabel("amplitude")
+            
+            #remove overlapping labels
+            ticks = axs[0].yaxis.get_major_ticks()
+            if len(ticks) : ticks[0].label1.set_visible(False)
+            if len(ticks) > 1 : ticks[1].label1.set_visible(False)
+            
+            fig.tight_layout()
+            
+            return fig
+        
         segments = segments.to_dict(orient="records")
         chunks = []
 
@@ -148,7 +194,24 @@ class ZooniversePipeline(Pipeline):
                     print("{} already exists, exportation skipped.".format(mp3))
                 else:
                     chunk_audio.export(mp3, format="mp3")
-
+                    
+                if self.spectro:
+                    png = os.path.join(self.destination, "chunks", chunk.getbasename("png"))
+                    
+                    #convert pydub sound data into raw data that the parselmouth library can use
+                    bit_depth = chunk_audio.sample_width * 8
+                    array_type = get_array_type(bit_depth)
+                    
+                    sound = array.array(array_type, chunk_audio._data)
+                    sr = chunk_audio.frame_rate
+                    fig = _create_spectrogram(sound,sr) #create the plot figure
+                    
+                    if os.path.exists(png) and os.path.getsize(png) > 0:
+                        print("{} already exists, exportation skipped.".format(png))
+                    else:
+                        fig.savefig(png)
+                    plt.close(fig)
+                        
                 chunks.append(chunk)
 
         return chunks
@@ -161,6 +224,7 @@ class ZooniversePipeline(Pipeline):
         segments: str,
         chunks_length: int = -1,
         chunks_min_amount: int = 1,
+        spectrogram: bool = False,
         profile: str = "",
         threads: int = 1,
         **kwargs
@@ -180,6 +244,8 @@ class ZooniversePipeline(Pipeline):
         :type chunks_length: int, optional
         :param chunks_min_amount: minimum amount of chunk per segment, defaults to 1
         :type chunks_min_amount: int, optional
+        :param spectrogram: the extraction generates a png spectrogram, defaults to False
+        :type spectrogram: bool, optional
         :param profile: recording profile to extract from. If undefined, raw recordings will be used.
         :type profile: str
         :param threads: amount of threads to run-on, defaults to 0
@@ -198,7 +264,8 @@ class ZooniversePipeline(Pipeline):
         self.project = ChildProject.projects.ChildProject(path)
 
         self.chunks_length = int(chunks_length)
-        self.chunks_min_amount = chunks_min_amount
+        self.chunks_min_amount = int(chunks_min_amount)
+        self.spectro = spectrogram
         self.profile = profile
 
         threads = int(threads)
@@ -238,6 +305,7 @@ class ZooniversePipeline(Pipeline):
                     "segment_offset": c.segment_offset,
                     "wav": c.getbasename("wav"),
                     "mp3": c.getbasename("mp3"),
+                    "png": c.getbasename("png") if self.spectro else "NA",
                     "date_extracted": datetime.datetime.now().strftime(
                         "%Y-%m-%d %H:%M:%S"
                     ),
@@ -318,6 +386,8 @@ class ZooniversePipeline(Pipeline):
             self.chunks,
             {"recording_filename", "onset", "offset", "uploaded", "mp3"},
         )
+        
+        from panoptes_client import Panoptes, Project, Subject, SubjectSet
 
         Panoptes.connect(username=self.zooniverse_login, password=self.zooniverse_pwd)
         zooniverse_project = Project(project_id)
@@ -423,6 +493,7 @@ class ZooniversePipeline(Pipeline):
         """
         self.get_credentials(zooniverse_login, zooniverse_pwd)
 
+        from panoptes_client import Panoptes, Project, Classification
         Panoptes.connect(username=self.zooniverse_login, password=self.zooniverse_pwd)
         project = Project(project_id)
 
@@ -515,6 +586,11 @@ class ZooniversePipeline(Pipeline):
             help="minimum amount of chunks to extract from a segment (default value: 1)",
             default=1,
         )
+        parser_extraction.add_argument(
+            "--spectrogram",
+            help="the extraction generates a png spectrogram (default False)",
+            action="store_true",
+        )
         parser_extraction.add_argument(
             "--segments", help="path to the input segments dataframe", required=True
         )
